==> mls-0.3.tar.gz: mls-0.3/src/panel_cmd.cpp <==
///	@file	panel_cmd.cpp
///	@brief	panel_cmd.cpp
#include <ncurses.h>
#include <sys/stat.h>

#include <string>
#include <vector>

#include "mls.h"
#include "panel.h"
#include "strutil.h"
#include "dialog.h"
#include "conf.h"
#include "commandimp.h"
#include "StringTokenizer.h"

using namespace std;
using namespace strutil;
using namespace MLS;

void MLS::CommandImp::Edit()
{
	Run(e_tConf.extern_editor + ' ' + addslash(panel->GetCurrentFile().name));
}

void MLS::CommandImp::View()
{
	Run(e_tConf.extern_viewer + ' ' + addslash(panel->GetCurrentFile().name));
}

///	@brief	모든 커맨드 메시지를 관리
///	@param	p	실행 시킬 명령어
///	@return		명령어 실행 결과
int MLS::CommandImp::ParseAndRun(const string &p)
{
	bool	bFlag  = false, 
		 	bPause = false,
		 	bParam = false,
		 	bMcd   = false ,
		 	bConfirm = false;
	
	File 	rf = panel->GetCurrentFile();
	string 	arg;

	// 파싱
	for (string::const_iterator i = p.begin(); i!= p.end(); i++)
	{
		if (*i == '%')
		{
			bFlag = true;
			continue;
		}
		
		if (bFlag)
		{
			switch(*i)
			{
			// 파일을 리턴해줌
			case '1':			
			case 'F':
				arg += addslash(rf.name);
				break;
				
			case 'N':
			{
				if (rf.name[0]=='.') return 0;
				
				string::size_type p = rf.name.rfind('.');
				
				if (p!=string::npos)
					arg += addslash(rf.name.substr(0, p-1));
				else
					arg += addslash(rf.name);
				break;
			}			
			
			case 'E':
				if (rf.ext().empty()) return 0;
				arg += addslash(rf.ext());
				
				break;
				
			case 'W':
				bPause = true;
				break;
				
			case '%':
				arg += '%';
				break;
				
			case 'S':
			{
				Selection tSelection = panel->GetSelection();
				
				if (tSelection.Size() == 0) return 0;				
				bool first = true;				
			
				for (int t = 0; t< tSelection.Size(); t++)
				{
					if (first) first= false;
					else arg += ' ';
					
					arg += addslash(tSelection[t]);
				}
				break;
			}
			
			case 's':
			{
				Selection tSelection = panel->GetSelection();
				
				if (!tSelection.Size() == 0)
				{
					bool first = true;					
				
					for (int t = 0; t< tSelection.Size(); t++)
					{
						if (first) first= false;
						else arg += ' ';
					
						arg += addslash(tSelection[t]);
					}
				}
				else 
				{
					if (rf.name == "..") return 0;
					arg += addslash(rf.name);
				}
				
				break;
			}
			
			case 'd':
			
				arg += addslash(panel->GetCurrentPath());
				break;		
			
			case 'D':
				bMcd = true;				
				break;
			
			case 'P':
				bParam= true;
			
				break;
				
			case 'H':
				arg += addslash(e_tConf.home);
				
				break;
				
			case 'Q':
				bConfirm = true;
				break;
				
			default:
				break;
			}
			
			bFlag = false;
			continue;
		}
		else arg += *i;
	}
	
	if (bMcd)
	{
		MLS::DirMcd mcd;
		mcd.setCur(panel->GetCurrentPath());
		if (!mcd.proc())
            return 0;
		arg += addslash(mcd.getCur());
	}
	
	if (bParam)	if (InputBox("Edit Parameter", arg)<0) return 0;
		
	if (bConfirm)
		if (YNBox("Do you want to run this operation?", YN_N) != YN_Y) return 0;
		
	// 실행
	int nRt = Run(arg, bPause);
	
	panel->Refresh();
	return nRt;
}

==> mls-0.4.tar.gz: mls-0.4/src/panel_cmd.cpp <==
///	@file	panel_cmd.cpp
///	@brief	panel_cmd.cpp
#include <ncurses.h>
#include <sys/stat.h>

#include <string>
#include <vector>

#include "mls.h"
#include "panel.h"
#include "strutil.h"
#include "mlslocale.h"
#include "dialog.h"
#include "conf.h"
#include "commandimp.h"
#include "StringTokenizer.h"

using namespace std;
using namespace strutil;
using namespace MLS;

void MLS::CommandImp::Edit()
{
	ParseAndRun(e_tConf.extern_editor + " %1");
}

void MLS::CommandImp::View()
{
	ParseAndRun(e_tConf.extern_viewer + " %1");
}

///	@brief	모든 커맨드 메시지를 관리
///	@param	p	실행 시킬 명령어
///	@return		명령어 실행 결과
int MLS::CommandImp::ParseAndRun(const string &p, bool Pause)
{
	bool	bFlag  = false, 
		 	bPause = Pause,
		 	bParam = false,
		 	bMcd   = false ,
		 	bConfirm = false,
			bBackground = false;
			
	// cd 구현
	if (p.substr(0, 2) == "cd")
	{
		string sStr;
		if (p == "cd" || p == "cd ") 
			sStr = panel->GetRealPath("~");
		else
			sStr = panel->GetRealPath(p.substr(3));
		
		// Zip일때는 실행하지 않는다.
		if (panel->isZip() == false)
		{
			LOG("chdir [%s]", sStr.c_str());
			int nRet = 0;
			if (sStr != "")	
			{
				if (panel->Read(sStr) == true) nRet = 1;
				panel->Refresh();
			}
			return nRet;
		}
	}
	
	File 	rf = panel->GetCurrentFile();
	string 	arg, tmp_name;
	
	// 압축파일인지 검사하여 /tmp 밑에 압축을 풀고 그 파일을 링크해준다.
	if (panel->isZip() == true)
	{
		try
		{
			if (panel->m_tReader->View(rf, "/tmp") == SUCCESS)
			{
				tmp_name = rf.name;
				rf.name = "/tmp/" + rf.FullName;
			}
		}
		catch(Exception& ex)
		{
			MsgBox(	gettext("ERROR"), gettext("file reading failure !!!!"));
			return 0;
		}
	}
	
	// 파싱
	for (string::const_iterator i = p.begin(); i!= p.end(); i++)
	{
		if (*i == '%')
		{
			bFlag = true;
			continue;
		}
		
		if (bFlag)
		{
			switch(*i)
			{
			// 파일을 리턴해줌
			case '1':
			case 'F':
				arg += addslash(rf.name);
				break;
				
			case 'N':
			{
				if (rf.name[0]=='.') return 0;
				
				string::size_type p = rf.name.rfind('.');
				
				if (p!=string::npos)
					arg += addslash(rf.name.substr(0, p-1));
				else
					arg += addslash(rf.name);
				break;
			}			
			
			case 'E':
				if (rf.ext().empty()) return 0;
				arg += addslash(rf.ext());
				
				break;
				
			case 'W':
				bPause = true;
				break;
				
			case '%':
				arg += '%';
				break;
				
			case 'S':
			{
				Selection tSelection = panel->GetSelection();
				
				if (tSelection.Size() == 0) return 0;				
				bool first = true;				
			
				for (int t = 0; t< tSelection.Size(); t++)
				{
					if (first) first= false;
					else arg += ' ';
					
					arg += addslash(tSelection[t]);
				}
				break;
			}
			
			case 's':
			{
				Selection tSelection = panel->GetSelection();
				
				if (!tSelection.Size() == 0)
				{
					bool first = true;					
				
					for (int t = 0; t< tSelection.Size(); t++)
					{
						if (first) first= false;
						else arg += ' ';
					
						arg += addslash(tSelection[t]);
					}
				}
				else 
				{
					if (rf.name == "..") return 0;
					arg += addslash(rf.name);
				}
				
				break;
			}
			
			case 'd':
			
				arg += addslash(panel->GetCurrentPath());
				break;		
			
			case 'D':
				bMcd = true;				
				break;
			
			case 'P':
				bParam= true;
				break;
				
			case 'H':
				arg += addslash(e_tConf.home);
				
				break;
				
			case 'Q':
				bConfirm = true;
				break;
				
			case 'B':
				bBackground = true;
				break;
				
			default:
				break;
			}
			
			bFlag = false;
			continue;
		}
		else arg += *i;
	}
	
	if (bMcd)
	{
		MLS::DirMcd mcd;
		mcd.setCur(panel->GetCurrentPath());
		if (!mcd.proc())
            return 0;
		arg += addslash(mcd.getCur());
	}
	
	if (bParam)	if (InputBox(gettext("Edit Parameter"), arg)<0) return 0;
		
	if (bConfirm)
		if (YNBox(gettext("Do you want to run this operation?"), YN_N) != YN_Y) return 0;
	
	// 실행
	int nRt = Run(arg, bPause, bBackground);
	
	// /tmp에 풀린 압축 파일을 지운다.
	if (panel->isZip() == true && bBackground == false) 
	{
		string sDir;
		sDir = rf.name.substr(5);
		
		string::size_type p = sDir.find('/');
		
		if (p != -1)
			sDir = "rm -rf /tmp/" + sDir.substr(0, p);
		else
			sDir = "rm -rf /tmp/" + sDir;
		
		system(sDir.c_str());
		rf.name = tmp_name;
	}
	panel->Refresh();
	return nRt;
}
